{
 "cells": [
  {
   "cell_type": "markdown",
   "id": "6c9cc0ce-154f-463e-b15e-ea8478ecb3e0",
   "metadata": {},
   "source": [
    "\n",
    "\n",
    "# Debugging\n",
    "\n",
    "> “The most effective debugging tool is still careful thought, coupled with judiciously placed print statements.” — Brian Kernighan, co-creator of Unix \n",
    "\n",
    "Some useful resources and nice histories:\n",
    "- Python debugging: <https://www.youtube.com/watch?v=a7qIcIaL4zs>\n",
    "- Interesting debugging stories:\n",
    "  - My car is allergic to vanilla icecream: <https://news.ycombinator.com/item?id=37584399>\n",
    "  - Bugs with real workd damage: <https://news.ycombinator.com/item?id=36114641>\n",
    "  - More examples (500 miles email): <http://beza1e1.tuxen.de/lore/index.html>\n",
    "  - The code worked differently when the moon was full: <https://news.ycombinator.com/item?id=28686016>\n",
    "  - Fun bugs: <https://news.ycombinator.com/item?id=15790812>\n",
    "  - Top 10 bugs found in C++ projects in 2021: <https://pvs-studio.com/en/blog/posts/cpp/0901/>\n",
    "  - Heisenbug: <https://textual.textualize.io/blog/2023/02/11/the-heisenbug-lurking-in-your-async-code/>\n",
    "- Book: <https://www.whyprogramsfail.com/>\n",
    "- How to Report Bugs Effectively : <https://www.chiark.greenend.org.uk/~sgtatham/bugs.html>\n",
    "- Debugging with GDB: <https://news.ycombinator.com/item?id=30512302>\n",
    "- Reverse engineering:  <https://github.com/hugsy/gef>\n",
    "- gdb breakpoint tricks: https://news.ycombinator.com/item?id=39170901\n",
    "- Spectre and meltdown: https://www.youtube.com/watch?v=YRohz9VO1YY\n",
    "\n",
    "## Examples to fix\n",
    "\n",
    "ICTP examples\n",
    "\n",
    "Go to\n",
    "[ICTP-examples](https://bitbucket.org/iluvatar/scientific-computing-part-01/src),\n",
    "then go to debugging, then ICTP-gdb<sub>examples</sub> , and download\n",
    "each file, or just download all files compressed at [example\n",
    "files](https://bitbucket.org/iluvatar/scientific-computing-part-01/get/master.tar.bz2)\n",
    "\n",
    "These are more examples: [original\n",
    "link](http://www.tutorialspoint.com/gnu_debugger/gdb_debugging_examples.htm)"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "3fafbe32",
   "metadata": {},
   "source": [
    "\n",
    "## Example\n",
    "\n",
    "Fix the following code (first try to predict the problems and then use your current skills to catch the bug)\n",
    "\n",
    "```cpp\n",
    "#include <iostream>\n",
    "#include <cstdlib>\n",
    "\n",
    "int main(int argc, char **argv)\n",
    "{\n",
    "  // declare variables\n",
    "  const int N = 10;\n",
    "  double * array;\n",
    "  //reserve memory\n",
    "  array = new double [N];\n",
    "  // initialise array\n",
    "  for (int ii = 0; ii < N; ++ii) {\n",
    "    array[ii] = 2*(++ii); // != 2*(ii++) ?\n",
    "  }\n",
    "  // print some values\n",
    "  int idx = 2;\n",
    "  std::cout << array[idx] << std::endl;\n",
    "  idx = 10;\n",
    "  std::cout << array[idx] << std::endl; // compiles, but ...\n",
    "\n",
    "  // free memory\n",
    "  //delete [] array;\n",
    "\n",
    "  return EXIT_SUCCESS;\n",
    "}\n",
    "```\n",
    "\n",
    "\n",
    "\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "8b4fdc43",
   "metadata": {},
   "source": [
    "\n",
    "## Sanitizers\n",
    "\n",
    "Besides the typical printing to help debugging, a first helping hand\n",
    "would the compiler sanitizers. Try recompiling the previous code using\n",
    "the sanitizer flags as\n",
    "\n",
    "``` shell\n",
    "g++ -fsanitize=address -fsanitize=leak -fsanitize=undefined source.cpp\n",
    "```\n",
    "\n",
    "and then run it\n",
    "\n",
    "``` shell\n",
    "./a.out\n",
    "```\n",
    "\n",
    "Fo you observe something new? sanitizers check your code in runtime and\n",
    "help you find errors faster.\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "8d2a9960",
   "metadata": {},
   "source": [
    "\n",
    "## GDB (Gnu debugger)\n",
    "\n",
    "### How to use gdb?\n",
    "\n",
    "Let's use the following c++ code, called `gdb_example_01.cpp`:\n",
    "\n",
    "``` cpp\n",
    "#include <iostream>\n",
    "#include <cstdlib>\n",
    "\n",
    "int main(int argc, char **argv)\n",
    "{\n",
    "  // declare variables\n",
    "  const int N = 10;\n",
    "  double * array;\n",
    "  //reserve memory\n",
    "  array = new double [N];\n",
    "  // initialise array\n",
    "  for (int ii = 0; ii < N; ++ii) {\n",
    "    array[ii] = 2*(++ii); // != 2*(ii++) ?\n",
    "  }\n",
    "  // print some values\n",
    "  int idx = 2;\n",
    "  std::cout << array[idx] << std::endl;\n",
    "  idx = 10;\n",
    "  std::cout << array[idx] << std::endl; // compiles, but ...\n",
    "\n",
    "  // free memory\n",
    "  delete [] array;\n",
    "\n",
    "  return EXIT_SUCCESS;\n",
    "}\n",
    "```\n",
    "\n",
    "To compile for debugging, you should use\n",
    "\n",
    "``` bash\n",
    "$ g++ -g -ggdb gdb_example_01.cpp -o gdb_example_01.x\n",
    "```\n",
    "\n",
    "The flag `-g` tells the compiler to generate additional debugging\n",
    "symbols to be used by the debugger. In addition, you can also specify\n",
    "`-ggdb` to be more specific about the debugger symbols you want\n",
    "(specific for gdb). Even more, you can also specify the debug level by\n",
    "using `-g3` (or `-ggdb3`). Default is 2. Level 3 allows for debugging fo\n",
    "preprocessor macros.\n",
    "\n",
    "Now, if you run your code\n",
    "\n",
    "``` bash\n",
    "$ ./gdb_example_01.x\n",
    "```\n",
    "\n",
    "you will get the original output. The code does not have any error, and\n",
    "in principle it does not need debugging. But, you can still use the\n",
    "debugger to explore what your \"correct\" code is doing.\n",
    "\n",
    "To attach `gdb` to your executable, run gdb as\n",
    "\n",
    "``` bash\n",
    "$ gdb ./gdb_example_01.x\n",
    "```\n",
    "\n",
    "> **NOTE:** gdb can also attach to already running process, by using the\n",
    "> pid. For more info, see `man gdb` and maybe `man ps` .\n",
    "\n",
    "This will bring to something similar to (this is for a Mountain Lion Mac\n",
    "os X machine)\n",
    "\n",
    "``` bash\n",
    "GNU gdb (GDB) 7.5.1\n",
    "Copyright (C) 2012 Free Software Foundation, Inc.\n",
    "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n",
    "This is free software: you are free to change and redistribute it.\n",
    "There is NO WARRANTY, to the extent permitted by law.  Type \"show copying\"\n",
    "and \"show warranty\" for details.\n",
    "This GDB was configured as \"x86_64-apple-darwin12.2.0\".\n",
    "For bug reporting instructions, please see:\n",
    "<http://www.gnu.org/software/gdb/bugs/>.\n",
    "(gdb)\n",
    "```\n",
    "\n",
    "> **NOTE** From now on, we will be **inside** the gdb console, so in the\n",
    "> next examples please write the commands inside the gdb console. For\n",
    "> example, `(gdb) step` means write `step` *inside* the gdb console, and\n",
    "> not `(gdb) step` inside the shell.\n",
    "\n",
    "Now, you are ready to go. First of all, keep in mind that your program\n",
    "is not yet running, it was just attached to gdb and all the debugging\n",
    "symbols were loaded. To actually run your program, you must write:\n",
    "\n",
    "``` bash\n",
    "(gdb) run\n",
    "```\n",
    "\n",
    "> **NOTE:** If your command needs command line arguments, you can run it\n",
    "> as `(gdb) run [arglist]`, where `[arglist]` is the optional argument\n",
    "> list.\n",
    "\n",
    "And now you will get … guess … `0` again. Your program just ran up to\n",
    "the end without any error. Now let's get to the interesting part.\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "81b243db",
   "metadata": {},
   "source": [
    "\n",
    "### Breakpoints\n",
    "\n",
    "A breakpoint is a point in your source code where you want to stop the\n",
    "execution in order to check whatever condition you want (the value of a\n",
    "variable, the change in a variable, etc), or, for example, to continue\n",
    "the execution by steps. We will set a breakpoint at line 5 of our source\n",
    "code. This is as simple as writting\n",
    "\n",
    "``` bash\n",
    "(gdb) break 5\n",
    "```\n",
    "\n",
    "and now run again the executable\n",
    "\n",
    "``` bash\n",
    "(gdb) run\n",
    "```\n",
    "\n",
    "As you can see, the code now runs up to line 5, where gdb inform us we\n",
    "have reached the breakpoint. If you check the source code, we are in the\n",
    "line\n",
    "\n",
    "``` bash\n",
    "int a;\n",
    "```\n",
    "\n",
    "that is, we are just declaring the variable `a`. What is its value at\n",
    "this point? 0? 1? Let's check\n",
    "\n",
    "``` bash\n",
    "(gdb) print a\n",
    "```\n",
    "\n",
    "The value should be poor memory garbage, like 0xff2345 or similar. It\n",
    "depends on the machine. Now, if we move one step to the next line in the\n",
    "execution order\n",
    "\n",
    "``` bash\n",
    "(gdb) next\n",
    "```\n",
    "\n",
    "we will be in the line `a = 0`. Let's check again the value of variable\n",
    "`a`\n",
    "\n",
    "``` bash\n",
    "(gdb) print a\n",
    "```\n",
    "\n",
    "it should now print 0. As we can see, we can check the value of a local\n",
    "variable. In general, we will be able to check the value of both local\n",
    "and global variables.\n",
    "\n",
    "Other examples for setting breakpoints:\n",
    "\n",
    "-   Setting the break point at a given line of a given file:\n",
    "\n",
    "``` bash\n",
    "(gdb) break gdb_example_XX.cpp:10\n",
    "```\n",
    "\n",
    "-   Setting a break point when calling a function:\n",
    "\n",
    "``` bash\n",
    "(gdb) break function_name\n",
    "```\n",
    "\n",
    "-   Checking all breakpoints\n",
    "\n",
    "``` bash\n",
    "(gdb) info breakpoints\n",
    "```\n",
    "\n",
    "-   Deleting a break point (the second one)\n",
    "\n",
    "``` bash\n",
    "(gdb) delete 2\n",
    "```\n",
    "\n",
    "-   Ignore a given breakpoint (`3`) for a number of times (`100`)\n",
    "    (useful for iterations)\n",
    "\n",
    "``` bash\n",
    "(gdb) ignore 3 100\n",
    "```\n",
    "\n",
    "-   Conditional break :\n",
    "\n",
    "``` bash\n",
    "(gdb) break gdb_example_XX.cpp:5 if localvar == 10\n",
    "```\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "8a339f5a",
   "metadata": {},
   "source": [
    "\n",
    "### Execution Control\n",
    "\n",
    "The instruction `next` tells gdb to advance in the execution order,\n",
    "which also executes completely functions and returns to the main\n",
    "function, while `step` goes inside the function step-by-step. Let's\n",
    "check the difference with another code (gdb<sub>example02</sub>.cpp):\n",
    "\n",
    "``` cpp\n",
    "#include <iostream>\n",
    "\n",
    "void do_something(int & var);\n",
    "\n",
    "int main(int argc, char **argv)\n",
    "{\n",
    "  int a;\n",
    "  a = 0;\n",
    "  std::cout << a << std::endl;\n",
    "  do_something(a);\n",
    "  std::cout << a << std::endl;\n",
    "\n",
    "  return EXIT_SUCCESS;\n",
    "}\n",
    "\n",
    "void do_something(int & var)\n",
    "{\n",
    "  int localvar = -1;\n",
    "  var = 10.0;\n",
    "}\n",
    "```\n",
    "\n",
    "Compile it with debugging symbols\n",
    "\n",
    "``` bash\n",
    "$ g++ -g -ggdb gdb_example_02.cpp -o gdb_example_02.x\n",
    "```\n",
    "\n",
    "and attach to gdb\n",
    "\n",
    "``` bash\n",
    "$ gdb ./gdb_example_02.x\n",
    "```\n",
    "\n",
    "Let's make a breakpoint at line 9 and then run the code\n",
    "\n",
    "``` bash\n",
    "(gdb) break gdb_example_02.cpp:9\n",
    "(gdb) run\n",
    "```\n",
    "\n",
    "Now advance step by step and explore the variables at local level. Try\n",
    "to check the difference between `next` and `step` exactly at the\n",
    "function call.\n",
    "\n",
    "``` bash\n",
    "(gdb) step\n",
    "(gdb) step\n",
    "(gdb) next\n",
    "(gdb) ...\n",
    "(gdb) print localvar\n",
    "(gdb) print &localvar\n",
    "```\n",
    "\n",
    "You can also change the output format when printing a variable:\n",
    "\n",
    "``` bash\n",
    "(gdb) p /d a\n",
    "(gdb) p /t a\n",
    "(gdb) p /x a\n",
    "```\n",
    "\n",
    "Add a break point at the function `do_something`. Compare the addresses\n",
    "of the variables `a` and `var` inside the function `do_something`. Are\n",
    "they the same or different? why?\n",
    "\n",
    "Other control structures:\n",
    "\n",
    "-   Watch for a change on a given variable:\n",
    "\n",
    "``` bash\n",
    "(gdb) watch foo\n",
    "```\n",
    "\n",
    "-   Read-Watch for a given variable:\n",
    "\n",
    "``` bash\n",
    "(gdb) rwatch foo\n",
    "```\n",
    "\n",
    "-   Read-Write-Watch for a given variable:\n",
    "\n",
    "``` bash\n",
    "(gdb) awatch foo\n",
    "```\n",
    "\n",
    "-   Continue until next breakpoint:\n",
    "\n",
    "``` bash\n",
    "(gdb) continue\n",
    "```\n",
    "\n",
    "-   Finish current function:\n",
    "\n",
    "``` bash\n",
    "(gdb) finish\n",
    "```\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "6cc797d5",
   "metadata": {},
   "source": [
    "\n",
    "### Other useful gdb commands\n",
    "\n",
    "-   `(gdb) bt` : Backtrace: display the program stack:\n",
    "-   `(gdb) edit [file:]function` : look at the program line where it is\n",
    "    presently stopped.\n",
    "-   `(gdb) help [name]` : Show information about GDB command name, or\n",
    "    general information about using GDB\n",
    "-   `(gdb) quit` : Exit from GDB.\n",
    "-   `(gdb) list [file:]function` : type the text of the program in the\n",
    "    vicinity of where it is presently stopped.\n",
    "-   `(gdb) call localfunction()` : Calls a function called\n",
    "    `localfunction()` linked in the code.\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "5d512db5",
   "metadata": {},
   "source": [
    "\n",
    "### Catching a segfault\n",
    "\n",
    "Sometimes, when you are lazy or sloppy with the memory you use, or when\n",
    "you just have bad luck (some strange problems), you could face the\n",
    "infamous `Segementation fault` error. Let's see how the debugger can\n",
    "help us to catch a segfault and to fix it!\n",
    "\n",
    "First, please compile the following code (`gdb_example_segfault.cpp`):\n",
    "\n",
    "``` c++\n",
    "#include <iostream>\n",
    "#include <cstdlib>\n",
    "\n",
    "int main(int argc, char **argv)\n",
    "{\n",
    "  // declare variables\n",
    "  const int N = 10;\n",
    "  double * array;\n",
    "  //reserve memory\n",
    "  array = new double [N];\n",
    "  // initialise array\n",
    "  for (int ii = 0; ii < N; ++ii) {\n",
    "    array[ii] = 2*(++ii); // != 2*(ii++) ?\n",
    "  }\n",
    "  // print some values\n",
    "  int idx = 2;\n",
    "  std::cout << array[idx] << std::endl;\n",
    "  idx = 10;\n",
    "  std::cout << array[idx] << std::endl; // compiles, but ...\n",
    "\n",
    "  // free memory\n",
    "  delete [] array;\n",
    "\n",
    "  return EXIT_SUCCESS;\n",
    "}\n",
    "```\n",
    "\n",
    "Now compile and run gdb on it:\n",
    "\n",
    "``` bash\n",
    "$ g++ -g gdb_example_segfault.cpp -o gdb_example_segfault.x\n",
    "$ gdb ./gdb_example_segfault.x\n",
    "(gdb) run\n",
    "```\n",
    "\n",
    "What happens? it runs, but then you are facing the terrible segfault\n",
    "error. What is the cause? let's use gdb:\n",
    "\n",
    "``` bash\n",
    "(gdb) where\n",
    "```\n",
    "\n",
    "Now it tell us that the problem is on line XXXXX. If we check our source\n",
    "code, that is the line where we are accessing element with index 10 on\n",
    "an array whose maximum index is 9 (since its size is 10), and therefore\n",
    "we are out the fence. Now comment or remove that line, recompile, and\n",
    "rerun on gdb. Is there any error?\n",
    "\n",
    "You can also check the value of the iteration variables or related to\n",
    "track better the error. But `where` is many times enough to point you to\n",
    "the problem.\n",
    "\n",
    "You can also test the effect of other variable value, by using `set`:\n",
    "\n",
    "``` bash\n",
    "(gdb) break 15\n",
    "(gdb) run\n",
    "...\n",
    "(gdb) set var idx = 4\n",
    "(gdb) step\n",
    "```\n",
    "\n",
    "NOTE: If you do not get aby error but just strange results, try\n",
    "re-compiling your software with the flag -fsanitize=address\n",
    "-fsanitize=leak (only works recent for gnu compilers) and then you will\n",
    "catch the errors at running time.\n",
    "\n",
    "1.  Printing an array with gdb\n",
    "\n",
    "    Please try the following, where `array` is the array name and `len`\n",
    "    is its lenght. Play with several values for `len`:\n",
    "\n",
    "    ``` bash\n",
    "    (gdb) print *array@len\n",
    "    ```\n",
    "\n",
    "    Do you want to know the array size?\n",
    "\n",
    "    ``` bash\n",
    "    (gdb) ptype array\n",
    "    ```\n",
    "\n",
    "Please debug this code\n",
    "\n",
    "``` cpp\n",
    "    #include <iostream>\n",
    "\n",
    "    int main(void)\n",
    "    {\n",
    "      //double x[10], y[5]; // imprime raro\n",
    "      double y[5], x[10]; // imprime 1\n",
    "\n",
    "      for(int ii = 0; ii < 5; ++ii) {\n",
    "        y[ii] = ii +1;\n",
    "      }\n",
    "      for(int ii = 0; ii < 10; ++ii) {\n",
    "        x[ii] = -(ii +1);\n",
    "      }\n",
    "\n",
    "      double z = x[10];\n",
    "      std::cout << z << std::endl;\n",
    "\n",
    "      return 0;\n",
    "    }\n",
    "\n",
    "    ```\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "f940aeeb",
   "metadata": {},
   "source": [
    "\n",
    "## Gdb Interfaces \n",
    "\n",
    "1.  Tui mode\n",
    "\n",
    "    gdb -tui mode: Ncurses gui\n",
    "\n",
    "2.  Seer: https://github.com/epasveer/seer\n",
    "  \n",
    "3.  Inside emacs\n",
    "\n",
    "    When having a file openned in emacs, press `M-x` and write gdb,\n",
    "    press enter. You will be presented with an example command to run\n",
    "    gdb, modify it as you wish. Then, emacs will be yor debugger, and\n",
    "    you can move from frame to frame by using the cursor or `C-x o` .\n",
    "\n",
    "    Please check [emacs gdb mode](http://www.linuxjournal.com/article/7876)\n",
    "\n",
    "    -   Open your source code with emacs.  \n",
    "    -   Compile with `M-x compile`. `M` is the meta function. By default it\n",
    "        uses make, but you can modify the compile command.\n",
    "    -   Activate de debugging: `M-x gdb`\n",
    "    -   Debug. To move from one frame to another you can use either the\n",
    "        mouse or the shortcut `ctrl-x o` . To close a pane, use `ctrl-x 0`\n",
    "        when inside that pane.  \n",
    "-   \n",
    "\n",
    "1.  gdbgui\n",
    "\n",
    "    -   <https://www.gdbgui.com/>\n",
    "    -   <https://www.youtube.com/watch?v=em842geJhfk&t=0s>\n",
    "\n",
    "2.  gdbfrontend\n",
    "\n",
    "    -   <https://oguzhaneroglu.com/projects/gdb-frontend/>\n",
    "    -   <https://github.com/rohanrhu/gdb-frontend>\n",
    "    -   <https://www.youtube.com/watch?v=6LNR8u19x6Y>\n",
    "\n",
    "3.  DDD : Data Display Debugger\n",
    "\n",
    "    Although dated, its great strenght is the ability to plot data with\n",
    "    the command `grap plot dataarrayname` , and check its evilution as\n",
    "    the you debug.\n",
    "\n",
    "4.  Others\n",
    "\n",
    "    Codeblocks, geany, clion, etc all them have interfaces for gdb.\n",
    "\n",
    "5.  Online gdb\n",
    "    <https://www.onlinegdb.com/>\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "2c81c74e",
   "metadata": {},
   "source": [
    "\n",
    "### Other gdb tips\n",
    "#### Poor man's profiler\n",
    "\n",
    "[Example with gdb](http://readwrite.com/2010/11/01/using-gdb-as-a-poor-mans-profi)\n",
    "\n",
    "-   Get the process id:\n",
    "\n",
    "``` bash\n",
    "$ top\n",
    "```\n",
    "\n",
    "or\n",
    "\n",
    "``` bash\n",
    "$ ps aux\n",
    "```\n",
    "\n",
    "-   Attach the process to gdb (`PID` is the process id):\n",
    "\n",
    "``` bash\n",
    "$ gdb -p PID\n",
    "```\n",
    "\n",
    "Now the program is paused.\n",
    "\n",
    "-   check the stack:\n",
    "\n",
    "``` bash\n",
    "(gdb) thread apply all bt\n",
    "```\n",
    "\n",
    "Check what appears first.\n",
    "\n",
    "-   Continue the execution:\n",
    "\n",
    "``` bash\n",
    "(gdb) continue\n",
    "```\n",
    "\n",
    "-   Repeat previous two steps as many times as you want and make a\n",
    "    histogram of the most executed functions.\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "fd98acdb",
   "metadata": {},
   "source": [
    "\n",
    "<!-- ### Modern tools\n",
    "\n",
    "1.  Python\n",
    "\n",
    "    Gdb version \\>= 7.x supports python. You can use to explore more\n",
    "    info about your program, pretty print data, and even plot (although\n",
    "    blocking).\n",
    "\n",
    "    To load a c++ var into a python do, inside gdb,\n",
    "\n",
    "    ``` shell\n",
    "    py my_array = gdb.parse_and_eval(\"my_array\")\n",
    "    py print my_array.type\n",
    "    ```\n",
    "\n",
    "    Even more, to plot some data structure with matplotlib, you can as\n",
    "    follows (Based on\n",
    "    <https://blog.semicolonsoftware.de/debugging-numerical-c-c-fortran-code-with-gdb-and-numpy/>)\n",
    "\n",
    "    ``` c\n",
    "    int main(int argc, char **argv) {\n",
    "      const int nrow = 4;\n",
    "      const int ncol = 3;\n",
    "      double x[nrow * ncol];\n",
    "      for(int i = 0; i < nrow; i++) {\n",
    "        for(int j = 0; j < ncol; j++) {\n",
    "      x[i * ncol + j]  = i * j;\n",
    "        }\n",
    "      }\n",
    "      // BREAK here\n",
    "    }\n",
    "    ```\n",
    "\n",
    "    Follow the previous url to install gdb<sub>numpy</sub>.py.\n",
    "\n",
    "    Compile as\n",
    "\n",
    "    ``` shell\n",
    "    gcc ex-01.c -g -ggdb\n",
    "    ```\n",
    "\n",
    "    and now, run `gdb` and put the following commands\n",
    "\n",
    "    ``` shell\n",
    "    br 10 # sets a breakpoint\n",
    "    run # to load the program symbols\n",
    "    py import gdb_numpy\n",
    "    py x = gdb_numpy.to_array(\"(double *)x\", shape=(4 * 3,)).reshape(4, 3)\n",
    "    py print x\n",
    "    py import matplotlib.pyplot as plt\n",
    "    py plt.imshow(x)\n",
    "    py plt.show()\n",
    "    ```\n",
    "\n",
    "    Use gdb to execute, step by step, the following code\n",
    "\n",
    "    ``` cpp\n",
    "    #include <iostream>\n",
    "    #include <cstdlib>\n",
    "\n",
    "    int main(int argc, char **argv)\n",
    "    {\n",
    "        int a;\n",
    "        a = 0;\n",
    "        std::cout << a << std::endl;\n",
    "        a = 5;\n",
    "\n",
    "        return EXIT_SUCCESS;\n",
    "    }\n",
    "\n",
    "    ``` -->\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "7750fa92",
   "metadata": {},
   "source": [
    "\n",
    "## DDD : Data Display Debugger\n",
    "\n",
    "The [Data Display Debugger - ddd](http://www.gnu.org/software/ddd/) is a\n",
    "graphical frontend for command line debuggers like gdb, dbx, jdb,\n",
    "bashdb, etc, and some modified python debugger. It easies the debug\n",
    "processing by adding some capabilities, like visual monitoring of data\n",
    "and data plotting (killer feature).\n",
    "\n",
    "![](http://www.gnu.org/software/ddd/all.png)\n",
    "![](http://www.gnu.org/software/ddd/plots.png)\n",
    "\n",
    "> **NOTE**: DDD is hard to configure, it is so rebel. If after\n",
    "> configuring some options you are having issues, like dd hanging with a\n",
    "> message \"Waiting for gdb to start\", or the plotting window is always\n",
    "> waiting, then close ddd, delete the config file with `rm -rf ~/.ddd`,\n",
    "> and start over by configuring ONLY one option at a time (configure one\n",
    "> option, then close it, then open, then configure other option, then\n",
    "> close, etc).\n",
    "\n",
    "### Exercise\n",
    "\n",
    "Repeat the previous debugging exercises by using ddd. In particular, use\n",
    "the `display` and `watch`. Do you find them convenient? and what about\n",
    "`graph display data`?\n",
    "\n",
    "### DDD data plotting with gnuplot\n",
    "\n",
    "One of the killer features of ddd is its capability to actually plot the\n",
    "data with gnuplot. Let's do an example:\n",
    "\n",
    "-   Compile (with debugging activated, of course) the code `array.cpp`.\n",
    "-   Now open the executable with ddd.\n",
    "-   Establish a break point at main.\n",
    "-   Run the executable under ddd.\n",
    "-   Now write `graph plot data`, or equivalently, you can select `data`\n",
    "    with the mouse and then click to plot on the menu bar. **NOTE** : If\n",
    "    you are having issues please try to configure gnuplot helper to be\n",
    "    on an external window, then close, then re-open, and try again.\n",
    "    Check also Notes above.\n",
    "-   Now proceed by steps. You will be able to see a plot of the array\n",
    "    `data` as it changes its values. Awesome, isn't it?\n",
    "-   Do the same for the 2d array. What will you expect now to be\n",
    "    plotted? a line or a surface?\n",
    "\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "2a56964a",
   "metadata": {},
   "source": [
    "\n",
    "## Valgrind\n",
    "\n",
    "![Valgrind Logo](http://valgrind.org/images/valgrind-100.png)\n",
    "\n",
    "[Valgrind official web page](http://valgrind.org/)\n",
    "\n",
    "From Web Page: \\> Valgrind is an instrumentation framework for building\n",
    "dynamic analysis tools. There are Valgrind tools that can automatically\n",
    "detect many memory management and threading bugs, and profile your\n",
    "programs in detail. You can also use Valgrind to build new tools.\n",
    "\n",
    "> The Valgrind distribution currently includes six production-quality\n",
    "> tools: a memory error detector, two thread error detectors, a cache\n",
    "> and branch-prediction profiler, a call-graph generating cache and\n",
    "> branch-prediction profiler, and a heap profiler. It also includes\n",
    "> three experimental tools: a heap/stack/global array overrun detector,\n",
    "> a second heap profiler that examines how heap blocks are used, and a\n",
    "> SimPoint basic block vector generator. It runs on the following\n",
    "> platforms: X86/Linux, AMD64/Linux, ARM/Linux, PPC32/Linux,\n",
    "> PPC64/Linux, S390X/Linux, MIPS/Linux, ARM/Android (2.3.x and later),\n",
    "> X86/Android (4.0 and later), X86/Darwin and AMD64/Darwin (Mac OS X\n",
    "> 10.6 and 10.7, with limited support for 10.8).\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "ab39b0f4",
   "metadata": {},
   "source": [
    "\n",
    "### Graphical Interfaces\n",
    "\n",
    "-   [Valkyrie](http://valgrind.org/downloads/guis.html)\n",
    "-   [kcachegrind/qcachegrind](http://kcachegrind.sourceforge.net/html/Home.html)\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "91a623e0",
   "metadata": {},
   "source": [
    "\n",
    "### As a memory checker\n",
    "\n",
    "In this section we will focus on valgrind as memory checker tool\n",
    "(debugging part). Therefore, we will invoke valgrind with that option:\n",
    "\n",
    "``` bash\n",
    "$ valgrind --tool=memcheck --leak-check=yes myprog arg1 arg2\n",
    "```\n",
    "\n",
    "where `myprog` is your executable's name, and `arg1` etc are the\n",
    "arguments.\n",
    "\n",
    "We will use the files inside `valgrind/Valgrind-QuickStart`\n",
    "\n",
    "Please compile the file `a.c` (which is also on the compressed file -\n",
    "check wiki main page) and run valgrind on it. What do you obtain? Do you\n",
    "have memory leaks? Please fix it until you have no memory errors.\n",
    "\n",
    "Do the same for `b.c` .\n",
    "\n",
    "1.  Example : Memory leak\n",
    "\n",
    "    ``` cpp\n",
    "    #include <iostream>\n",
    "    #include <cstdlib>\n",
    "\n",
    "    void getmem(double *ptr, int n);\n",
    "\n",
    "    int main(int argc, char **argv)\n",
    "    {\n",
    "      // declare variables\n",
    "      const int N = 10000;\n",
    "      double * array;\n",
    "      for (int ii = 0 ; ii < 20; ++ii) {\n",
    "        getmem(array, N);\n",
    "      }\n",
    "      // free memory\n",
    "      // delete [] array;\n",
    "\n",
    "      return EXIT_SUCCESS;\n",
    "    }\n",
    "\n",
    "    void getmem(double *ptr, int n)\n",
    "    {\n",
    "      ptr = new double [n];\n",
    "      data[0] = data[1]*2;\n",
    "    }\n",
    "\n",
    "    ```\n",
    "\n",
    "2.  More examples : ICTP\n",
    "\n",
    "    <https://github.com/thehackerwithin/PyTrieste/tree/master/valgrind>\n",
    "\n",
    "    Please enter the `SoftwareCarpentry-ICTP` valgrind subfolder and\n",
    "    locate the source files. Compile `simpleTest.cc` as\n",
    "\n",
    "    ``` bash\n",
    "    $ g++ -g simpleTest.cc -o simpleTest\n",
    "    ```\n",
    "\n",
    "    Now run valgrind on it\n",
    "\n",
    "    ``` bash\n",
    "    $ valgrind --track-origins=yes --leak-check=full ./simpleTest 300 300\n",
    "    ```\n",
    "\n",
    "    Check how the memory leaks depend on the parameters. Fix the code.\n",
    "\n",
    "## <span class=\"todo TODO\">TODO</span> Homework\n",
    "\n",
    "Modularize one of the previous codes into a header a implementation file\n",
    "and a main file. This will help when writing tests."
   ]
  },
  {
   "cell_type": "markdown",
   "id": "c4e7b88d",
   "metadata": {},
   "source": []
  }
 ],
 "metadata": {
  "language_info": {
   "name": "python"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 5
}
